function [ys,check] = HANKDynamics_steadystate(ys,exo)

% Computes stationary equilibrium of the HANK model for Dynare; format is required
% to be called by Dynare (follows example of NK_baseline.mod in Dynare examples)
%
% Daeha Cho, December, 2022

%tStart = tic;
%fprintf('\nComputing steady state...\n')

%----------------------------------------------------------------
% Call parameters 
%----------------------------------------------------------------
%setParameters;


%----------------------------------------------------------------
% Read in parameters from Dynare declaration
%----------------------------------------------------------------

% Initialize indicator
check = 0;

% Read parameters from Dynare
global M_ 

% Read out parameters to access them with their name
for iParameter = 1:M_.param_nbr
  paramname = deblank(M_.param_names(iParameter,:));
  eval(['global ' paramname]);
  eval([ paramname ' = M_.params(' int2str(iParameter) ');']);
end



displayOpt = 'off';       % 'iter-detailed' or 'off'

%----------------------------------------------------------------
% Save values of steady state variables for Dynare (must be exactly
% as declared in Dynare)
%----------------------------------------------------------------
%----------------------------------------------------------------
% Call steady state values
%----------------------------------------------------------------
load('steadyState_Decision_Distribution');
load('steadyStateAggregates');

%----------------------------------------------------------------
% Coefficients on conditional expectation function
%----------------------------------------------------------------

for iEpsilon = 1 : nEpsilon
	for iAsset = 1 : nAssets
		eval(sprintf('expectationCoefficient_%d_%d = mCoefficients(iEpsilon,iAsset);',...
			iEpsilon,iAsset));
	end
end


%----------------------------------------------------------------
% Density of households away from borrowing constraint
%----------------------------------------------------------------

% Moments and parameters of density away from borrowing constraint
for iEpsilon = 1 : nEpsilon
	for iMoment = 1 : nMeasure
		eval(sprintf('moment_%d_%d = mMoments(iEpsilon,iMoment);',iEpsilon,iMoment));
		eval(sprintf('measureCoefficient_%d_%d = mParameters(iEpsilon,iMoment+1);',iEpsilon,iMoment));
	end
end

% Consumption moments
for iEpsilon = 1 : nEpsilon
	eval(sprintf('momentC_%d_1 = mMomentsC(iEpsilon);',iEpsilon));
end

%----------------------------------------------------------------
% Mass at borrowing constraint
%----------------------------------------------------------------
for iEpsilon = 1 : nEpsilon
	eval(sprintf('mHat_%d = mHat(iEpsilon);',iEpsilon));
end




%----------------------------------------------------------------
% Epsilon transition matrix
%----------------------------------------------------------------
for iEpsilon = 1 : nEpsilon
    for iEpsilonPrime = 1 : nEpsilon	
        eval(sprintf('epsilonTransition_%d_%d = mEpsilonTransitionSS(iEpsilon,iEpsilonPrime);',...
			iEpsilon,iEpsilonPrime));
    end
end


%----------------------------------------------------------------
% Mass of idiosyncratic shocks
%----------------------------------------------------------------
for iEpsilon = 1 : nEpsilon
	eval(sprintf('epsilonMass_%d = vEpsilonInvariant(iEpsilon);',iEpsilon)); 
end


%----------------------------------------------------------------
% Quantities 
%----------------------------------------------------------------
c = cSS;
cL = cLSS;
if modelOption_ ~= 3
cH = cHSS;
else 
cH = (cSS - (1-Hshare)*cLSS)/Hshare;
end
inv = invSS;
g = gSS;
gtilde = 1/(1-(gSS/ySS));
k = kSS;
uk = 1;
d = dSS;
y = ySS;
x = xSS;
u = uSS;
ua = uaSS;
n = nSS;
theta = thetaSS;
lambda = lambdaSS;
f = fSS; 
rhox = rhoxSS;
v = vSS;
bg = bgSS;
 

%----------------------------------------------------------------
% Prices 
%----------------------------------------------------------------
R = RSS;
Pi = PiSS;
pStar = pStarSS;
h1P = 1/(1-xxiP*(muuSS/(RSS/PiSS)));
h2P = h1P; 
DeltaP = DeltaPSS;
mc = mcSS;
rk = rkSS;
q = 1;
w = wSS;
ttau = ttauSS;

%----------------------------------------------------------------
% Law of motion for aggregate TFP
%----------------------------------------------------------------
etaP = etaPSS;
muu = muuSS;
upsilon = 1;
A = 1;
zeta = 1;
zetaR = 1;
PiStar = PiSS;
rhoxtilde = 0; 

%----------------------------------------------------------------
% Auxiliary variables of interest 
%----------------------------------------------------------------

xHat = 0; 
cHat = 0;
cLHat = 0;
cHHat = 0;
invHat = 0;
nHat = 0;
PiHat = 0;
vHat = 0;


%----------------------------------------------------------------
% Observables
%----------------------------------------------------------------

obsgX = 0;
obsgC = 0;
obsgI = 0;
obsgG = 0;
obsF = 0;
obsS = 0;
obsgWE = 0;
obsPi = 0;
obsFF = 0;
obsU = 0;



% Save endogenous variables back into ys
for ii = 1 : M_.orig_endo_nbr
  varname = deblank(M_.endo_names(ii,:));
  eval(['ys(' int2str(ii) ') = ' varname ';']); 
end

%fprintf('... Done!  Elapsed time: %2.2f seconds \n\n',toc(tStart))
