% Creates grids to use in various approximations
% 
% Thomas Winberry, July 26th, 2016

%---------------------------------------------------------------
% Grids for approximating individual decisions rules
%---------------------------------------------------------------

if splineOpt == 0	% if using polynomials and approximating conditional expectation

	global vAssetsGridZeros vAssetsGrid mEmploymentGrid mAssetsGrid mEmploymentPrimeGrid

	% Create nAssets Chebyshev nodes in the interval [-1,1]
	vAssetsGridZeros = -cos(((2 * (1:nAssets)-1)' * pi) / (2 * nAssets));

	% Scale up nodes in [-1,1] to state space [assetsMin, assetsMax]
	vAssetsGrid = scaleUp(vAssetsGridZeros,assetsMin,assetsMax); %vAssetsGrid is column
	
else	% if using splines and approximating asset accumulation

	global vAssetsGrid mEpsilonGrid mAssetsGrid mEpsilonPrimeGrid

	% Grid over assets
	vAssetsGrid = exp(linspace(log(assetsMin + .01),log(assetsMax + .01),nAssets)');
	vAssetsGrid = vAssetsGrid - .01;
	
end
	
% Make matrix versions of the grids
mEmploymentGrid = repmat(vEmploymentGrid,[1 nAssets]);  
mAssetsGrid = repmat(vAssetsGrid',[nEpsilon 1]);  
mEmploymentPrimeGrid = repmat(vEmploymentGrid,[1 nEpsilon*nAssets]); 

%---------------------------------------------------------------
% Fine grid, for histogram and plotting functions
%---------------------------------------------------------------

global vAssetsGridFine vAssetsGridFineZeros mEmploymentGridFine mAssetsGridFine mEmploymentPrimeGridFine 

% Assets grid - bins for historgram
vAssetsGridFine = linspace(assetsMin,assetsMax,nAssetsFine)';

% Scale down to [-1,1]
vAssetsGridFineZeros = scaleDown(vAssetsGridFine,assetsMin,assetsMax); %column vector

% Make matrix versions of grids
mEmploymentGridFine = repmat(vEmploymentGrid,[1 nAssetsFine]); % what is this for?
mAssetsGridFine = repmat(vAssetsGridFine',[nEpsilon 1]);
mEmploymentPrimeGridFine = repmat(vEmploymentGrid,[1 nStateFine]); 

%---------------------------------------------------------------
% Quadrature grid, to integrate density (away from borrowing constraint)
%---------------------------------------------------------------

global vQuadratureWeights vAssetsGridQuadratureZeros vAssetsGridQuadrature mEmploymentGridQuadrature ...
	mAssetsGridQuadrature

% Compute grid in the interval [-1, 1]
[vAssetsGridQuadratureZeros,vQuadratureWeights] = computeGaussLegendreQuadrature(nAssetsQuadrature);

% Scale up grid
vAssetsGridQuadrature = scaleUp(vAssetsGridQuadratureZeros,assetsMin+1e-6,assetsMax);

% Make matrix versions of the grids
mEmploymentGridQuadrature = repmat(vEmploymentGrid,[1 nAssetsQuadrature]); 
mAssetsGridQuadrature = repmat(vAssetsGridQuadrature', [nEpsilon 1]);
