% Sets parameter values 
%
% Daeha Cho, December, 2022



global bbetaLInit ssigma aaBar RSS PiSS bbetaH Hshare Hshare_T MPCH muuSS aalpha ddelta ss2prime rrhouk ppsi etaPSS...
    xxiP iiotaP pStarSS h1PSS h2PSS DeltaPSS ggamma nSS fSS rhoxSS rrhoxBar wSS kkappa MMBar bInit xxiW iiotaW...
    mcSS rkSS kSS lambdaSS uSS uaSS thetaSS vSS FF dSS xSS ttauSS ySS invSS cSS gSS gtildeSS bgSS...
    rrhoR pphiPi pphiX...
    vEmploymentGrid vEpsilonGrid mEpsilonTransitionSS vEpsilonInvariant...
	targetMPC cUOvercEMean...
    rrhoetaP rrhomuu rrhoupsilon rrhoTFP rrhozeta rrhorhox rrhopiStar rrhog... 
    ssigmaetaP ssigmamuu ssigmaupsilon ssigmaTFP ssigmaR ssigmazeta ssigmarhox...
    ssigmapiStar ssigmag ssigmaW;
	

%----------------------------------------------------------------
% Estimated parameters 
%----------------------------------------------------------------


ssigmaetaP = paramMean(1);
ssigmamuu = paramMean(2);
ssigmaupsilon = paramMean(3);
ssigmaR = paramMean(4);
ssigmazeta = paramMean(5);
ssigmarhox = paramMean(6);   
ssigmag = paramMean(7);
ssigmaW = paramMean(8);
ssigmaTFP = paramMean(9);


ss2prime = paramMean(10); 
ppsi = paramMean(11);
xxiP = paramMean(12);
iiotaP = paramMean(13);
xxiW = paramMean(14);
iiotaW = paramMean(15);

rrhoR = paramMean(16);
pphiPi = paramMean(17);
pphiX = paramMean(18);
rrhoetaP = paramMean(19);
rrhomuu = paramMean(20);
rrhoupsilon = paramMean(21);
rrhozeta = paramMean(22);
rrhorhox = paramMean(23);
rrhog = paramMean(24);
rrhoTFP = paramMean(25);

%----------------------------------------------------------------
% Fixed parameters 
%----------------------------------------------------------------

% Preferences
bbetaLInit = 0.9779;										% discount factor (impatient)
ssigma = 2;											% coefficient of relative risk aversion
aaBar = 0;                                              % borrowing constraint

RSS = 1+ (1.58/100);                                % quartlerly nominal interest rate
PiSS = 1 + 0.9583/100;
muuSS = 1.0047; 
bbetaH = 1/(RSS/(PiSS*muuSS^ssigma));                   % discount factor (patient)
Hshare = 0.1;
MPCH = (1-bbetaH*muuSS^(1-ssigma));  
Hshare_T = 0.8; 

% Technology
aalpha = .33;										% capital share
ddelta = 0.02;										% depreciation rate (annual calibration)
rkSS = RSS/PiSS - 1 + ddelta; %real rental price of capital 
rrhouk = rkSS;     % capital utilization cost parameter

% Goods Markets
etaPSS = 0.2;                                          % elasticity of substitution
pStarSS = 1;
h1PSS = 1/(1-xxiP*(muuSS/(RSS/PiSS)));
h2PSS = h1PSS; 
DeltaPSS = 1;

% Labor markets
ggamma = 0.5;    % matching function elasticity  
nSS = 0.94;   % employment rate

x0 = [0.592, 0.08, 1.5, 0.0694, 0.71]';

f = @(x0) calibrateLMParameters(x0);
options = optimoptions('fsolve','TolFun',1e-9); 
[xvar,err,exitflag] = fsolve(f,x0,options);
disp(['Errors: ' num2str(max(abs(err)))]);

fSS = xvar(1); % job finidng rate
rhoxSS = xvar(2); % job separation rate 
wSS = xvar(3); % real wage
kkappa = xvar(4); % vacancy posting cost
MMBar = xvar(5); % matching efficiency 

rrhoxBar = log(1/rhoxSS - 1); %parameter of job separation rate function 


bInit = 0.3262;  % replacement rate


mcSS =  1/(1+etaPSS);  % real marginal cost
kSS = (rkSS/(aalpha*mcSS*nSS^(1-aalpha)))^(1/(aalpha-1)); % aggregate capital
lambdaSS = MMBar^(1/(1-ggamma))*fSS^(-ggamma/(1-ggamma));  % job filling rate
uSS = 1-nSS;   % unemployment rate
uaSS = uSS + rhoxSS*nSS;  % active searcher
thetaSS = (lambdaSS/MMBar)^(-1/ggamma); % tightness
vSS = thetaSS*uaSS;  % vacancy
FF = kSS^aalpha * nSS^(1-aalpha) * (1-mcSS); % fixed cost
ySS = (kSS^aalpha * nSS^(1-aalpha) - FF)/DeltaPSS;
xSS = ySS - kkappa * vSS ;      % GDP
invSS = kSS * (1- (1-ddelta)/muuSS);
gSS = xSS * 0.2;
gtildeSS = 1/(1-(gSS/ySS));
cSS =  xSS - gSS - invSS;
dSS = ySS - wSS*nSS  - kkappa * vSS - invSS;  % dividends to households



% Fiscal policy
bgSS = xSS * 4* 0.26;          % liquid assets  
ttauSS = (bInit * wSS * uSS  + gSS + (RSS/(PiSS*muuSS)-1) * bgSS)/(wSS*nSS + bInit*wSS*uSS);  

vEmploymentGrid = [0; 1];
vEpsilonGrid = vEmploymentGrid; 
mEpsilonTransitionSS = [1-fSS, fSS;...
                        rhoxSS*(1-fSS), (1-rhoxSS*(1-fSS))];
                                      
vEpsilonInvariant = [1 - nSS; nSS];

%Targets needed for calibration
targetMPC = 0.21;
cUOvercEMean = 1-0.23/(1-Hshare);


rrhopiStar = 0.95;
ssigmapiStar = 0.01;


%----------------------------------------------------------------
% Set approximation parameters
%----------------------------------------------------------------

global nEpsilon nAssets nState assetsMin assetsMax nAssetsFine nStateFine nAssetsQuadrature nStateQuadrature ...
	nMeasure nMeasureCoefficients maxIterations tolerance dampening splineOpt displayOpt
	
% Whether approximating decision rule with splines or polynomials
splineOpt = 0;	% if splineOpt = 1, use splines to approximate savings policy; if splineOpt = 0, use polynomials
				% to approximate conditional expectation function

% Whether to print out results from steady state computation
displayOpt = 'iter-detailed';       % 'iter-detailed' or 'off'

% Order of approximation
nEpsilon = 2;
nAssets = 25; % # of gridpoints in spline approximation or # of polynomials (npsi) in polynomial approximation
nState = nEpsilon * nAssets; % # of individual state variables

% Bounds on grid space
assetsMin = aaBar;	assetsMax = 1.5;

% Finer grid for analyzing policy functions
nAssetsFine = 100;
nStateFine = nEpsilon * nAssetsFine;

% Approximation of distribution
nMeasure = 4; % The number moments which we use to approximate for the pdf (ng)
nAssetsQuadrature = 20;  %The order of the Gauss-Legendre quadrature (mg)
nStateQuadrature = nEpsilon * nAssetsQuadrature;
nMeasureCoefficients = nEpsilon * nMeasure;

% Iteration on individual decisions
maxIterations = 2e4;
tolerance = 1e-5;
dampening = .95;