% Estimates HANK, RANK, and TANK 
%
% Daeha Cho, December, 2022

clear all
close all
clc


setpath;
cd('./Auxiliary Functions');

%----------------------------------------------------------------
% Set parameters
%----------------------------------------------------------------
load('HANKparam')
setParameters;


%----------------------------------------------------------------
% Compute approximation tools
%----------------------------------------------------------------

% Grids
computeGrids;

% Polynomials over grids
computePolynomials;

%----------------------------------------------------------------
% Save parameters in .mat files to import into Dynare 
%----------------------------------------------------------------

load('steadyState_Decision_Distribution');

    
% Economic parameters
save economicParameters.mat bbetaL ssigma aaBar bbetaH Hshare Hshare_T aalpha ddelta ss2prime rrhouk ppsi ...
     xxiP iiotaP ggamma rrhoxBar kkappa MMBar bb iiotaW xxiW FF rrhoR pphiPi pphiX vEpsilonGrid ...
     rrhoetaP rrhomuu rrhoupsilon rrhoTFP rrhozeta rrhorhox rrhopiStar rrhog... 
     ssigmaetaP ssigmamuu ssigmaupsilon ssigmaTFP ssigmaR ssigmazeta ssigmarhox...
     ssigmapiStar ssigmag ssigmaW;
    
% Steady state aggregate variables
save steadyStateAggregates.mat RSS PiSS muuSS nSS fSS rhoxSS wSS etaPSS DeltaPSS pStarSS h1PSS h2PSS...
     mcSS rkSS kSS lambdaSS uSS uaSS thetaSS vSS xSS bgSS ttauSS dSS ySS invSS gSS gtildeSS cSS cHSS cLSS cH_TSS cL_TSS...
     mEpsilonTransitionSS vEpsilonInvariant

% Approximation parameters
save approximationParameters.mat nEpsilon nAssets nState assetsMin assetsMax nAssetsFine nStateFine nAssetsQuadrature nStateQuadrature ...
	 nMeasure nMeasureCoefficients maxIterations tolerance dampening

% Grids
save grids.mat vAssetsGridZeros vAssetsGrid vEmploymentGrid vEpsilonGrid mEmploymentGrid mAssetsGrid mEmploymentPrimeGrid vAssetsGridFine ...
	 vAssetsGridFineZeros mEmploymentGridFine mAssetsGridFine mEmploymentPrimeGridFine vQuadratureWeights ...
	 vAssetsGridQuadratureZeros vAssetsGridQuadrature mEmploymentGridQuadrature mAssetsGridQuadrature
		
% Polynomials
save polynomials.mat vAssetsPoly vAssetsPolySquared vAssetsPolyFine vAssetsPolyQuadrature vAssetsPolyBC
	


%----------------------------------------------------------------
%Estimate HANK (produces Tables 3 and A.2)
%----------------------------------------------------------------
modelOption=1; 
dynare HANKDynamics.mod -Dmodel_option=1 -Dexer_option=1

delete *_dynamic.m
delete *_static.m
delete *.log
delete *_set_auxiliary_variables.m
delete *_results.mat
delete *.eps
delete *.asv


paramMean = oo_.posterior.metropolis.mean;
load('HANKDynamics_mh_mode')
paramMode = xparam1;

save HANKparam paramMean paramMode

estimationResults_HANK = oo_ ;
save estimationResults_HANK estimationResults_HANK;

delete HANKDynamics.m
rmdir HANKDynamics s


delete *_pindx.mat
delete *_params.mat
delete *_mode.mat
delete *_mean.mat



%----------------------------------------------------------------
%Estimate RANK and TANK (produces Tables 3 and 8)
%----------------------------------------------------------------

%Estimate RANK
modelOption=1; 
dynare RANKDynamics.mod -Dmodel_option=1 -Dexer_option=1


delete *_dynamic.m
delete *_static.m
delete *.log
delete *_set_auxiliary_variables.m
delete *_results.mat
delete *.eps
delete *.asv


paramMean = oo_.posterior.metropolis.mean;
load('RANKDynamics_mh_mode')
paramMode = xparam1;

save RANKparam paramMean paramMode

estimationResults_RANK = oo_ ;
save estimationResults_RANK estimationResults_RANK;

delete RANKDynamics.m

delete *_pindx.mat
delete *_params.mat
delete *_mode.mat
delete *_mean.mat

%Estimate TANK
modelOption=2; 
dynare RANKDynamics.mod -Dmodel_option=2 -Dexer_option=1


delete *_dynamic.m
delete *_static.m
delete *.log
delete *_set_auxiliary_variables.m
delete *_results.mat
delete *.eps
delete *.asv


paramMean = oo_.posterior.metropolis.mean;
load('RANKDynamics_mh_mode')
paramMode = xparam1;

save TANKparam paramMean paramMode

estimationResults_TANK = oo_ ;
save estimationResults_TANK estimationResults_TANK;

delete RANKDynamics.m


delete *_pindx.mat
delete *_params.mat
delete *_mode.mat
delete *_mean.mat



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Autocorrelatons (Figure 5)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

load('estimationResults_HANK')
load('estimationData')

covHANK = {estimationResults_HANK.PosteriorTheoreticalMoments.dsge.covariance.Mean};
corrHANK = {estimationResults_HANK.PosteriorTheoreticalMoments.dsge.correlation.Mean};  

load('estimationResults_RANK')

covRANK = {estimationResults_RANK.PosteriorTheoreticalMoments.dsge.covariance.Mean};
corrRANK = {estimationResults_RANK.PosteriorTheoreticalMoments.dsge.correlation.Mean}; 

varlist = {'obsgC', 'obsgI', 'obsgG', 'obsF', 'obsS', 'obsgWE', 'obsPi', 'obsFF'}; 

titlelist =...
{'dC_t,dC_{t-k} (Consumption growth)','dC_t,dI_{t-k}', 'dC_t,dG_{t-k}','dC_t,f_{t-k}','dC_t,s_{t-k}','dC_t,dW_{t-k}','dC_t,\pi_{t-k}','dC_t,R_{t-k}';...
'dI_t,dC_{t-k}','dI_t,dI_{t-k} (Investment growth)','dI_t,dG_{t-k}','dI_t,f_{t-k}','dI_t,s_{t-k}','dI_t,dW_{t-k}','dI_t,\pi_{t-k}','dI_t,R_{t-k}';...
'dG_t,dC_{t-k}','dG_t,dI_{t-k}', 'dG_t,dG_{t-k} (G purchase growth)','dG_t,f_{t-k}','dG_t,s_{t-k}','dG_t,dW_{t-k}','dG_t,\pi_{t-k}','dG_t,R_{t-k}';...
'f_t,dC_{t-k}','f_t,dI_{t-k}','f_t,dG_{t-k}','f_t,f_{t-k} (Job-finding rate)','f_t,s_{t-k}','f_t,dW_{t-k}','f_t,\pi_{t-k}','f_t,R_{t-k}';...
's_t,dC_{t-k}','s_t,dI_{t-k}','s_t,dG_{t-k}','s_t,f_{t-k}','s_t,s_{t-k} (Job-loss rate)','s_t,dW_{t-k}','s_t,\pi_{t-k}','s_t,R_{t-k}';...
'dW_t,dC_{t-k}','dW_t,dI_{t-k}','dW_t,dG_{t-k}','dW_t,f_{t-k}','dW_t,s_{t-k}','dW_t,dW_{t-k} (Wage inflation)','dW_t,\pi_{t-k}','dW_t,R_{t-k}';...
'\pi_t,dC_{t-k}','\pi_t,dI_{t-k}','\pi_t,dG_{t-k}','\pi_t,f_{t-k}','\pi_t,s_{t-k}','\pi_t,dW_{t-k}','\pi_t,\pi_{t-k} (Price inflation)','\pi_t,R_{t-k}';...
'R_t,dC_{t-k}','R_t,dI_{t-k}','R_t,dG_{t-k}','R_t,f_{t-k}','R_t,s_{t-k}','R_t,dW_{t-k}','R_t,\pi_{t-k}','R_t,R_{t-k} (Interest rate)'};

FigHandle = figure('Position', [150, 150, 1200, 600]);
set(0,'defaultlinelinewidth',1.5)

time = [0:5]';

for iplot = 1:length(varlist)
  
        [dataCorr, dataCorrL, dataCorrU] = computeCrossCorrelation(eval(varlist{iplot}),eval(varlist{iplot}),5);

        PlotValues = [[getfield(covRANK{1},varlist{iplot},varlist{iplot})/...
                   sqrt(getfield(covRANK{1},varlist{iplot},varlist{iplot})*getfield(covRANK{1},varlist{iplot},varlist{iplot}));...
                   getfield(corrRANK{1},varlist{iplot},varlist{iplot})];...
                   [getfield(covHANK{1},varlist{iplot},varlist{iplot})/...
                   sqrt(getfield(covHANK{1},varlist{iplot},varlist{iplot})*getfield(covHANK{1},varlist{iplot},varlist{iplot}));...
                   getfield(corrHANK{1},varlist{iplot},varlist{iplot})];...
                   dataCorrL; dataCorr; dataCorrU];
        yBounds1 = [min(PlotValues(:)),max(PlotValues(:))];
        
        subaxis(2,round(length(varlist)/2),iplot,'spacing',0.08), 
               plot(time,[getfield(covHANK{1},varlist{iplot},varlist{iplot})/...
                   sqrt(getfield(covHANK{1},varlist{iplot},varlist{iplot})*getfield(covHANK{1},varlist{iplot},varlist{iplot}));...
                   getfield(corrHANK{1},varlist{iplot},varlist{iplot})],'k:',...
                   time,[getfield(covRANK{1},varlist{iplot},varlist{iplot})/...
                   sqrt(getfield(covRANK{1},varlist{iplot},varlist{iplot})*getfield(covRANK{1},varlist{iplot},varlist{iplot}));...
                   getfield(corrRANK{1},varlist{iplot},varlist{iplot})],'r-.',...
                   time, dataCorrL, 'b--',time, dataCorr, 'b-',time, dataCorrU, 'b--');   
        title(titlelist(iplot,iplot),'FontSize',12); xlim([0 5]);ylim(yBounds1)
end



set(gcf,'color','w');
lg = legend({'HANK','RANK'},'Interpreter','latex'); 
set(lg,'FontSize',15);
legend('boxoff');
lg.Orientation = 'horizontal';
set(lg,'position',[0,0,1,0.08]);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Variance decomposition (Figure 3)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load('estimationResults_HANK')
load('estimationResults_RANK')

MEIGDPContribution_HANK = estimationResults_HANK.variance_decomposition(:,3);
STFPGDPContribution_HANK = estimationResults_HANK.variance_decomposition(:,4);
estimationResults_HANK.variance_decomposition(:,3) = STFPGDPContribution_HANK; 
estimationResults_HANK.variance_decomposition(:,4) = MEIGDPContribution_HANK;

MEIContribution_RANK = estimationResults_RANK.variance_decomposition(:,3);
STFPContribution_RANK = estimationResults_RANK.variance_decomposition(:,4);
estimationResults_RANK.variance_decomposition(:,3) = STFPContribution_RANK; 
estimationResults_RANK.variance_decomposition(:,4) = MEIContribution_RANK;

FigHandle = figure('Position', [150, 150, 1600, 800]);

defaultColor = [0 0.4470 0.7410;
    0.85 0.3250 0.098;
    0.9290 0.6940 0.1250;
    0.4940 0.1840 0.5560;
    0.4660 0.6740 0.1880;
    0.6350 0.0780 0.1840;
    0.3010 0.7450 0.9330];
set(0,'defaultAxesColorOrder',defaultColor)

newcolors = [defaultColor;
             0.7 0.7 0.7];
set(0,'DefaultAxesColorOrder',newcolors)

subaxis(2,2,1,'spacing',0.1);
x = [1 2];
y = [estimationResults_RANK.variance_decomposition(1,:);estimationResults_HANK.variance_decomposition(1,:)];
barh(x,y,'stacked','EdgeColor','none')
axis tight
yticklabels({'RANK','HANK'})
set(gca,'yticklabels',yticklabels,'FontSize',12)
title({'GDP growth'},'Interpreter','latex','FontSize',15)

subaxis(2,2,2,'spacing',0.1);
x = [1 2];
y = [estimationResults_RANK.variance_decomposition(2,:);estimationResults_HANK.variance_decomposition(2,:)];
barh(x,y,'stacked','EdgeColor','none')
axis tight
yticklabels({'RANK','HANK'})
set(gca,'yticklabels',yticklabels,'FontSize',12)
title({'Consumption growth'},'Interpreter','latex','FontSize',15)

subaxis(2,2,3,'spacing',0.1);
x = [1 2];
y = [estimationResults_RANK.variance_decomposition(3,:);estimationResults_HANK.variance_decomposition(3,:)];
barh(x,y,'stacked','EdgeColor','none')
axis tight
yticklabels({'RANK','HANK'})
set(gca,'yticklabels',yticklabels,'FontSize',12)
title({'Investment growth'},'Interpreter','latex','FontSize',15)

subaxis(2,2,4,'spacing',0.1);
x = [1 2];
y = [estimationResults_RANK.variance_decomposition(4,:);estimationResults_HANK.variance_decomposition(4,:)];
barh(x,y,'stacked','EdgeColor','none')
axis tight
yticklabels({'RANK','HANK'})
set(gca,'yticklabels',yticklabels,'FontSize',12)
title({'Government purchase growth'},'Interpreter','latex','FontSize',15)
set(gcf,'color','w');

lg = legend('Markup','Non-stat. tech.','Stat. tech.', 'MEI', 'Mon. policy','Preference', 'Job-separation', 'Gov. purchase'); 
set(lg,'FontSize',11);
legend('boxoff');
lg.Orientation = 'horizontal';
set(lg,'position',[0,0,1,0.06]);



cd('../')