// Dynare shell which solves the RANK model for computing IRFs
//
// Daeha Cho, December, 2022

//----------------------------------------------------------------
// Load parameters
//----------------------------------------------------------------

// Load in files containing parameters
economicParameters = load('economicParameters');
approximationParameters = load('approximationParameters');
steadyStateAggregates = load('steadyStateAggregates');
grids = load('grids');
polynomials = load('polynomials');

// Define economic parameters
parameters bbetaL ssigma aaBar bbetaH Hshare aalpha ddelta ss2prime rrhouk ppsi
	xxiP iiotaP ggamma rrhoxBar kkappa MMBar bb iiotaW xxiW FF rrhoR pphiPi pphiX
    rrhoetaP rrhomuu rrhoupsilon rrhoTFP rrhozeta rrhorhox rrhopiStar rrhog
    ssigmaetaP ssigmamuu ssigmaupsilon ssigmaTFP ssigmaR ssigmazeta ssigmarhox 
    ssigmapiStar ssigmag ssigmaW;


//Load in their values
@#define nEconomicParameters = 41
for iParam = 1 : @{nEconomicParameters}
	parameterName = deblank(M_.param_names(iParam,:));
	if isfield(economicParameters,parameterName)
		M_.params(iParam) = eval(['economicParameters.' parameterName]);
	end
end

// Define steady state aggregate variables
parameters RSS PiSS muuSS nSS fSS wSS mcSS rkSS kSS lambdaSS uSS uaSS thetaSS vSS rhoxSS
           etaPSS pStarSS h1PSS h2PSS DeltaPSS xSS bgSS ttauSS gSS gtildeSS dSS ySS invSS cSS cHSS cLSS;


//Load in their values
@#define nSteadyStateAggregates = 31
for iParam = 1 : @{nSteadyStateAggregates}
	parameterName = deblank(M_.param_names(@{nEconomicParameters}+iParam,:));
	if isfield(steadyStateAggregates,parameterName)
		M_.params(@{nEconomicParameters}+iParam) = eval(['steadyStateAggregates.' parameterName]);
	end
end

@#define nCounter = nEconomicParameters + nSteadyStateAggregates


//----------------------------------------------------------------
// Define variables
//----------------------------------------------------------------

@#include "variablesBlock_RANK.mod"

//----------------------------------------------------------------
// Model equations
//----------------------------------------------------------------

model;

@#include "equationsBlock_RANK.mod"

end;

//----------------------------------------------------------------
// 4. Computation
//----------------------------------------------------------------

// Specify shock process

shocks;
    var epsetaP; stderr 1;
    var epsmuu; stderr 1;
    var epsupsilon; stderr 1;
    var epsTFP; stderr 1;
    var epsR; stderr 1;
    var epszeta; stderr 1;
    var epsrhox; stderr 1;
    var epsg; stderr 1;
    var epsW; stderr 1;
end;

options_.steadystate.nocheck = 1;
//resid(1);


// Simulate
stoch_simul(order=1, irf=20, nocorr, nograph);
//cHat invHa t xHat nHat obsgX obsgC obsgI obsgG obsF obsS obsgWE obsPi obsFF; 



