function [ys,check] = RANKDynamics_steadystate(ys,exo)

% Computes stationary equilibrium of RANK for Dynare; format is required
% to be called by Dynare (follows example of NK_baseline.mod in Dynare examples)
%
% Daeha Cho, December, 2022

%tStart = tic;
%fprintf('\nComputing steady state...\n')

%----------------------------------------------------------------
% Call parameters 
%----------------------------------------------------------------
%setParameters;


%----------------------------------------------------------------
% Read in parameters from Dynare declaration
%----------------------------------------------------------------

% Initialize indicator
check = 0;

% Read parameters from Dynare
global M_ 

% Read out parameters to access them with their name
for iParameter = 1:M_.param_nbr
  paramname = deblank(M_.param_names(iParameter,:));
  eval(['global ' paramname]);
  eval([ paramname ' = M_.params(' int2str(iParameter) ');']);
end



displayOpt = 'off';       % 'iter-detailed' or 'off'

%----------------------------------------------------------------
% Save values of steady state variables for Dynare (must be exactly
% as declared in Dynare)
%----------------------------------------------------------------
%----------------------------------------------------------------
% Call steady state values
%----------------------------------------------------------------
load('steadyStateAggregates');


%----------------------------------------------------------------
% Quantities 
%----------------------------------------------------------------
c = cSS;
cH = cSS;
inv = invSS;
g = gSS;
gtilde = 1/(1-(gSS/ySS));
k = kSS;
uk = 1;
d = dSS;
y = ySS;
x = xSS;
u = uSS;
ua = uaSS;
n = nSS;
theta = thetaSS;
lambda = lambdaSS;
f = fSS; 
rhox = rhoxSS;
v = vSS;
bg = bgSS;
 

%----------------------------------------------------------------
% Prices 
%----------------------------------------------------------------
R = RSS;
Pi = PiSS;
pStar = pStarSS;
h1P = 1/(1-xxiP*(muuSS/(RSS/PiSS)));
h2P = h1P; 
DeltaP = DeltaPSS;
mc = mcSS;
rk = rkSS;
q = 1;
w = wSS;
ttau = ttauSS;

%----------------------------------------------------------------
% Law of motion for aggregate TFP
%----------------------------------------------------------------
etaP = etaPSS;
muu = muuSS;
upsilon = 1;
A = 1;
zeta = 1;
PiStar = PiSS;
rhoxtilde = 0; 

%----------------------------------------------------------------
% Auxiliary variables of interest 
%----------------------------------------------------------------

xHat = 0; 
cHat = 0;
invHat = 0;
nHat = 0;
PiHat = 0;
fHat = 0;


%----------------------------------------------------------------
% Observables
%----------------------------------------------------------------

obsgX = 0;
obsgC = 0;
obsgI = 0;
obsgG = 0;
obsF = 0;
obsS = 0;
obsgWE = 0;
obsPi = 0;
obsFF = 0;
obsU = 0;




% Save endogenous variables back into ys
for ii = 1 : M_.orig_endo_nbr
  varname = deblank(M_.endo_names(ii,:));
  eval(['ys(' int2str(ii) ') = ' varname ';']); 
end

%fprintf('... Done!  Elapsed time: %2.2f seconds \n\n',toc(tStart))
