// Dynare shell which generates counterfactual paths of HANK, Constant risk, RANK, and TANK models 
// given the sequence of shocks
//
// Daeha Cho, December, 2022


//----------------------------------------------------------------
// Load parameters
//----------------------------------------------------------------

@#include "parametersBlock.mod"

//----------------------------------------------------------------
// Define variables
//----------------------------------------------------------------

@#include "variablesBlock.mod"

//----------------------------------------------------------------
// Model equations
//----------------------------------------------------------------

model;
@#if model_option==1
    @#include "equationsBlock.mod"
@#endif

@#if model_option==2
    @#include "equationsBlock_constantRisk.mod"
@#endif


//--------------------------------------------------------------------------
// RANK economy 
//--------------------------------------------------------------------------
[name = 'RANK economy']    
1 = bbetaH * (R_R/(Pi_R(1)*muu(1)^ssigma)) * (zeta(1)/zeta) * (cH_R(1)/cH_R)^(-ssigma);



d_R = y_R - w_R * n_R  - kkappa * v_R - inv_R - (rrhouk*(1-ppsi)*(uk_R^(1/(1-ppsi))-1)) * (k_R(-1)/muu);  % dividends to households

n_R = (1-rhox) * n_R(-1) + MMBar * v_R * theta_R^(-ggamma); 

ua_R = u_R(-1) + rhox * n_R(-1);
 
u_R = (1-f_R) * (u_R(-1) + rhox * n_R(-1));

theta_R = v_R/ua_R;

u_R = 1 - n_R;
 
lambda_R = MMBar * theta_R^(-ggamma);



kkappa/lambda_R =  (A^(1-aalpha) * (1-aalpha) * (uk_R*k_R(-1))^aalpha * n_R^(-aalpha) * mc_R - w_R)
                    + (muu(1)/(R_R/Pi_R(1))) * (1-rhox(1)) * kkappa/lambda_R(1);

q_R =  (1/(R_R/Pi_R(1))) * (rk_R(1)*uk_R(1) + q_R(1) * (1-ddelta) - (rrhouk*(1-ppsi)*(uk_R(1)^(1/(1-ppsi))-1)));

rk_R = rrhouk * uk_R^(ppsi/(1-ppsi));

1 = q_R * upsilon * (1 - (ss2prime/2) * ((inv_R*muu)/inv_R(-1) - muuSS)^2 - ss2prime * ((inv_R*muu)/inv_R(-1)-muuSS) * ((inv_R*muu)/inv_R(-1)))
    + ss2prime * (1/(R_R/Pi_R(1))) * q_R(1) * upsilon(1) * ((inv_R(1)*muu(1))/inv_R - muuSS) * ((inv_R(1)*muu(1))/inv_R)^2;

k_R = (1 - ddelta) * (k_R(-1)/muu) + upsilon * (1 - (ss2prime/2) * ((inv_R*muu)/inv_R(-1)-muuSS)^2) * inv_R;



DeltaP_R * y_R = (uk_R*k_R(-1))^aalpha * (A * n_R)^(1-aalpha) - FF; 

pStar_R = h1P_R/h2P_R; 
h1P_R = (1+etaP) * mc_R + xxiP * muu(1) * (1/(R_R/Pi_R(1))) * h1P_R(1);
h2P_R = 1 + xxiP * muu(1) * (1/(R_R/Pi_R(1))) * ((Pi_R^iiotaP * PiSS^(1-iiotaP))/Pi_R(1)) * h2P_R(1);
1 = (1-xxiP) * pStar_R^(-1/etaP) + xxiP * ((Pi_R(-1)^iiotaP * PiSS^(1-iiotaP))/Pi_R)^(-1/etaP);
DeltaP_R = (1-xxiP)*pStar_R^(-((1+etaP)/etaP)) + xxiP * ((Pi_R(-1)^iiotaP * PiSS^(1-iiotaP))/Pi_R)^(-((1+etaP)/etaP))*DeltaP_R(-1);



rk_R = aalpha * (uk_R*k_R(-1))^(aalpha-1) * (A * n_R)^(1-aalpha) * mc_R;

w_R = (w_R(-1)/(Pi_R*muu))^iiotaW * (wwBar * (n_R/nSS)^xxiW)^(1-iiotaW);

bg_R + ttau_R * (w_R * n_R + bb * w_R * u_R) = bb*w_R*u_R + g_R + (R_R(-1)/(Pi_R*muu)) * (bg_R(-1));

bg_R = bgSS;

g_R = (1 - (1/gtilde)) * y_R;

log(R_R/RSS) = rrhoR*log(R_R(-1)/RSS) + (1-rrhoR)*( pphiPi*log(Pi_R/PiStar) + pphiX*log(x_R/xSS) ) + ssigmaR*epsR;

//log(R_R/RSS) = rrhoR*log(R_R(-1)/RSS) + (1-rrhoR)*( pphiPi*((log(Pi_R/PiStar)+log(Pi_R(-1)/PiStar)+log(Pi_R(-2)/PiStar)+log(Pi_R(-3)/PiStar))/4) 
//+ pphiX*( (log((x_R*muu)/(x_R(-1)*muuSS))+log((x_R(-1)*muu(-1))/(x_R(-2)*muuSS))+log((x_R(-2)*muu(-2))/(x_R(-3)*muuSS)) + log((x_R(-3)*muu(-3))/(x_R(-4)*muuSS)))/4) )
//+ ssigmaR*epsR;



c_R = cH_R;  

x_R = c_R + inv_R + g_R;



y_R = c_R + inv_R + g_R + kkappa * v_R + (rrhouk*(1-ppsi)*(uk_R^(1/(1-ppsi))-1)) * (k_R(-1)/muu);



obsgX_R = log(x_R) - log(x_R(-1)) + log(muu/muuSS);
obsgC_R = log(c_R) - log(c_R(-1)) + log(muu/muuSS);
obsgI_R = log(inv_R) - log(inv_R(-1)) + log(muu/muuSS);
obsF_R = f_R - fSS;
obsS_R = rhox*(1-f_R)- rhoxSS*(1-fSS);
obsgWE_R = log(w_R) - log(w_R(-1)) + log(muu/muuSS) + log(Pi_R/PiSS) + ssigmaW*epsW;
obsPi_R = log(Pi_R/PiSS);
obsFF_R = log(R_R/RSS);
obsU_R = u_R - uSS;



xHat_R = log(x_R/xSS); 
cHat_R = log(c_R/cSS);
invHat_R = log(inv_R/invSS);
nHat_R = n_R - nSS;
PiHat_R = log(Pi_R/PiSS);


//--------------------------------------------------------------------------
// TANK economy 
//--------------------------------------------------------------------------
[name = 'TANK economy']    
1 = bbetaH * (R_T/(Pi_T(1)*muu(1)^ssigma)) * (zeta(1)/zeta) * (cH_T(1)/cH_T)^(-ssigma);

cL_T = ((1-ttau_T) * w_T * (n_T + bb * u_T));


d_T = y_T - w_T * n_T  - kkappa * v_T - inv_T - (rrhouk*(1-ppsi)*(uk_T^(1/(1-ppsi))-1)) * (k_T(-1)/muu);  % dividends to households

n_T = (1-rhox) * n_T(-1) + MMBar * v_T * theta_T^(-ggamma); 

ua_T = u_T(-1) + rhox * n_T(-1);
 
u_T = (1-f_T) * (u_T(-1) + rhox * n_T(-1));

theta_T = v_T/ua_T;

u_T = 1 - n_T;
 
lambda_T = MMBar * theta_T^(-ggamma);



kkappa/lambda_T =  (A^(1-aalpha) * (1-aalpha) * (uk_T*k_T(-1))^aalpha * n_T^(-aalpha) * mc_T - w_T)
                    + (muu(1)/(R_T/Pi_T(1))) * (1-rhox(1)) * kkappa/lambda_T(1);

q_T =  (1/(R_T/Pi_T(1))) * (rk_T(1)*uk_T(1) + q_T(1) * (1-ddelta) - (rrhouk*(1-ppsi)*(uk_T(1)^(1/(1-ppsi))-1)));

rk_T = rrhouk * uk_T^(ppsi/(1-ppsi));

1 = q_T * upsilon * (1 - (ss2prime/2) * ((inv_T*muu)/inv_T(-1) - muuSS)^2 - ss2prime * ((inv_T*muu)/inv_T(-1)-muuSS) * ((inv_T*muu)/inv_T(-1)))
    + ss2prime * (1/(R_T/Pi_T(1))) * q_T(1) * upsilon(1) * ((inv_T(1)*muu(1))/inv_T - muuSS) * ((inv_T(1)*muu(1))/inv_T)^2;

k_T = (1 - ddelta) * (k_T(-1)/muu) + upsilon * (1 - (ss2prime/2) * ((inv_T*muu)/inv_T(-1)-muuSS)^2) * inv_T;



DeltaP_T * y_T = (uk_T*k_T(-1))^aalpha * (A * n_T)^(1-aalpha) - FF; 

pStar_T = h1P_T/h2P_T; 
h1P_T = (1+etaP) * mc_T + xxiP * muu(1) * (1/(R_T/Pi_T(1))) * h1P_T(1);
h2P_T = 1 + xxiP * muu(1) * (1/(R_T/Pi_T(1))) * ((Pi_T^iiotaP * PiSS^(1-iiotaP))/Pi_T(1)) * h2P_T(1);
1 = (1-xxiP) * pStar_T^(-1/etaP) + xxiP * ((Pi_T(-1)^iiotaP * PiSS^(1-iiotaP))/Pi_T)^(-1/etaP);
DeltaP_T = (1-xxiP)*pStar_T^(-((1+etaP)/etaP)) + xxiP * ((Pi_T(-1)^iiotaP * PiSS^(1-iiotaP))/Pi_T)^(-((1+etaP)/etaP))*DeltaP_T(-1);



rk_T = aalpha * (uk_T*k_T(-1))^(aalpha-1) * (A * n_T)^(1-aalpha) * mc_T;

w_T = (w_T(-1)/(Pi_T*muu))^iiotaW * (wwBar * (n_T/nSS)^xxiW)^(1-iiotaW);

bg_T + ttau_T * (w_T * n_T + bb * w_T * u_T) = bb*w_T*u_T + g_T + (R_T(-1)/(Pi_T*muu)) * (bg_T(-1));

bg_T = bgSS;

g_T = (1 - (1/gtilde)) * y_T;

log(R_T/RSS) = rrhoR*log(R_T(-1)/RSS) + (1-rrhoR)*( pphiPi*log(Pi_T/PiStar) + pphiX*log(x_T/xSS) ) + ssigmaR*epsR;

//log(R_T/RSS) = rrhoR*log(R_T(-1)/RSS) + (1-rrhoR)*( pphiPi*((log(Pi_T/PiStar)+log(Pi_T(-1)/PiStar)+log(Pi_T(-2)/PiStar)+log(Pi_T(-3)/PiStar))/4) 
//+ pphiX*( (log((x_T*muu)/(x_T(-1)*muuSS))+log((x_T(-1)*muu(-1))/(x_T(-2)*muuSS))+log((x_T(-2)*muu(-2))/(x_T(-3)*muuSS)) + log((x_T(-3)*muu(-3))/(x_T(-4)*muuSS)))/4) )
//+ ssigmaR*epsR;



c_T = Hshare_T * cH_T + (1 - Hshare_T) * cL_T;    

x_T = c_T + inv_T + g_T;

y_T = c_T + inv_T + g_T + kkappa * v_T + (rrhouk*(1-ppsi)*(uk_T^(1/(1-ppsi))-1)) * (k_T(-1)/muu);



obsgX_T = log(x_T) - log(x_T(-1)) + log(muu/muuSS);
obsgC_T = log(c_T) - log(c_T(-1)) + log(muu/muuSS);
obsgI_T = log(inv_T) - log(inv_T(-1)) + log(muu/muuSS);
obsF_T = f_T - fSS;
obsS_T = rhox*(1-f_T)- rhoxSS*(1-fSS);
obsgWE_T = log(w_T) - log(w_T(-1)) + log(muu/muuSS) + log(Pi_T/PiSS) + ssigmaW*epsW;
obsPi_T = log(Pi_T/PiSS);
obsFF_T = log(R_T/RSS);
obsU_T = u_T - uSS;



xHat_T = log(x_T/xSS); 
cHat_T = log(c_T/cSS);
invHat_T = log(inv_T/invSS);
nHat_T = n_T - nSS;
PiHat_T = log(Pi_T/PiSS);


end;

//----------------------------------------------------------------
// 4. Computation
//----------------------------------------------------------------

// Specify shock process

shocks;
    var epsetaP; stderr 1;
    var epsmuu; stderr 1;
    var epsupsilon; stderr 1;
    var epsTFP; stderr 1;
    var epsR; stderr 1;
    var epszeta; stderr 1;
    var epsrhox; stderr 1;
    var epsg; stderr 1;
    var epsW; stderr 1;
end;

options_.steadystate.nocheck = 1;
//resid(1);


// Compute steady state (nocheck option ensures that Dynare runs even if steady
// state only computed approximately, i.e., with small numerical error)
//steady(nocheck);

// Check regularity conditions (turn on to check)
//check;
//model_diagnostics;
//model_info;



// Simulate
stoch_simul(order=1, irf=0, nocorr, nograph);
 //xHat cHat cLHat cHHat invHat nHat PiHat fHat d;

// Feed in shocks
smoothedShocks2 = smoothedShocks(2:end,:);
shocksmatrix = smoothedShocks2;

%initialvalues=[oo_.dr.ys];
initialvalues=[firstSmoothedValues];
y2=simult_(initialvalues, oo_.dr, shocksmatrix, 1);
outcome = y2(:,1:end)'; 


