function [ys,check] = counterFactuals_steadystate(ys,exo)

% Computes stationary equilibrium of HANK, Constant risk, RANK, and TANK models for Dynare; format is required
% to be called by Dynare (follows example of NK_baseline.mod in Dynare examples)
%
% Daeha Cho, December, 2022

%tStart = tic;
%fprintf('\nComputing steady state...\n')

%----------------------------------------------------------------
% Call parameters 
%----------------------------------------------------------------
%setParameters;


%----------------------------------------------------------------
% Read in parameters from Dynare declaration
%----------------------------------------------------------------

% Initialize indicator
check = 0;

% Read parameters from Dynare
global M_ 

% Read out parameters to access them with their name
for iParameter = 1:M_.param_nbr
  paramname = deblank(M_.param_names(iParameter,:));
  eval(['global ' paramname]);
  eval([ paramname ' = M_.params(' int2str(iParameter) ');']);
end



displayOpt = 'off';       % 'iter-detailed' or 'off'

%----------------------------------------------------------------
% Save values of steady state variables for Dynare (must be exactly
% as declared in Dynare)
%----------------------------------------------------------------
%----------------------------------------------------------------
% Call steady state values
%----------------------------------------------------------------
load('steadyState_Decision_Distribution');
load('steadyStateAggregates');

%----------------------------------------------------------------
% Coefficients on conditional expectation function
%----------------------------------------------------------------

for iEpsilon = 1 : nEpsilon
	for iAsset = 1 : nAssets
		eval(sprintf('expectationCoefficient_%d_%d = mCoefficients(iEpsilon,iAsset);',...
			iEpsilon,iAsset));
	end
end


%----------------------------------------------------------------
% Density of households away from borrowing constraint
%----------------------------------------------------------------

% Moments and parameters of density away from borrowing constraint
for iEpsilon = 1 : nEpsilon
	for iMoment = 1 : nMeasure
		eval(sprintf('moment_%d_%d = mMoments(iEpsilon,iMoment);',iEpsilon,iMoment));
		eval(sprintf('measureCoefficient_%d_%d = mParameters(iEpsilon,iMoment+1);',iEpsilon,iMoment));
	end
end

% Consumption moments
for iEpsilon = 1 : nEpsilon
	eval(sprintf('momentC_%d_1 = mMomentsC(iEpsilon);',iEpsilon));
end

%----------------------------------------------------------------
% Mass at borrowing constraint
%----------------------------------------------------------------
for iEpsilon = 1 : nEpsilon
	eval(sprintf('mHat_%d = mHat(iEpsilon);',iEpsilon));
end




%----------------------------------------------------------------
% Epsilon transition matrix
%----------------------------------------------------------------
for iEpsilon = 1 : nEpsilon
    for iEpsilonPrime = 1 : nEpsilon	
        eval(sprintf('epsilonTransition_%d_%d = mEpsilonTransitionSS(iEpsilon,iEpsilonPrime);',...
			iEpsilon,iEpsilonPrime));
    end
end


%----------------------------------------------------------------
% Mass of idiosyncratic shocks
%----------------------------------------------------------------
for iEpsilon = 1 : nEpsilon
	eval(sprintf('epsilonMass_%d = vEpsilonInvariant(iEpsilon);',iEpsilon)); 
end


%----------------------------------------------------------------
% Quantities 
%----------------------------------------------------------------
c = cSS;
cL = cLSS; 
cH = cHSS;
inv = invSS;
g = gSS;
gtilde = 1/(1-(gSS/ySS));
k = kSS;
uk = 1;
d = dSS;
y = ySS;
x = xSS;
u = uSS;
ua = uaSS;
n = nSS;
theta = thetaSS;
lambda = lambdaSS;
f = fSS; 
rhox = rhoxSS;
v = vSS;
bg = bgSS;
TC = 0;
 

%----------------------------------------------------------------
% Prices 
%----------------------------------------------------------------
R = RSS;
Pi = PiSS;
pStar = pStarSS;
h1P = 1/(1-xxiP*(muuSS/(RSS/PiSS)));
h2P = h1P; 
DeltaP = DeltaPSS;
mc = mcSS;
rk = rkSS;
q = 1;
w = wSS;
ttau = ttauSS;

%----------------------------------------------------------------
% Law of motion for aggregate TFP
%----------------------------------------------------------------
etaP = etaPSS;
muu = muuSS;
upsilon = 1;
A = 1;
zeta = 1;
zetaR = 1;
PiStar = PiSS;
rhoxtilde = 0; 

%----------------------------------------------------------------
% Auxiliary variables of interest 
%----------------------------------------------------------------

xHat = 0; 
cHat = 0;
cLHat = 0;
cHHat = 0;
invHat = 0;
nHat = 0;
PiHat = 0;
vHat = 0;

%----------------------------------------------------------------
% Observables
%----------------------------------------------------------------

obsgX = 0;
obsgC = 0;
obsgI = 0;
obsgG = 0;
obsF = 0;
obsS = 0;
obsgWE = 0;
obsPi = 0;
obsFF = 0;
obsU = 0;


%----------------------------------------------------------------
% RANK variables
%----------------------------------------------------------------

c_R = cSS; 
cH_R = cSS;
inv_R = invSS;
k_R = kSS;
uk_R = 1;
d_R = dSS;
y_R = ySS;
x_R = xSS;
u_R = uSS;
ua_R = uaSS;
n_R = nSS;
theta_R = thetaSS;
lambda_R = lambdaSS; 
f_R = fSS; 
v_R = vSS; 
bg_R = bgSS; 
R_R = RSS;
Pi_R = PiSS; 
pStar_R = pStarSS;
h1P_R = 1/(1-xxiP*(muuSS/(RSS/PiSS)));
h2P_R = h1P_R; 
DeltaP_R = DeltaPSS; 
mc_R = mcSS; 
rk_R = rkSS;
q_R = 1;
w_R = wSS;
ttau_R = ttauSS; 
g_R = gSS;

obsgX_R = 0;
obsgC_R = 0;
obsgI_R = 0;
obsF_R = 0;
obsS_R = 0;
obsgWE_R = 0;
obsPi_R = 0;
obsFF_R = 0;
obsU_R = 0;

xHat_R = 0; 
cHat_R = 0;
invHat_R = 0;
nHat_R = 0;
PiHat_R = 0;

%----------------------------------------------------------------
% TANK variables
%----------------------------------------------------------------

c_T = cSS; 
cH_T = cH_TSS;
cL_T = cL_TSS;
inv_T = invSS;
k_T = kSS;
uk_T = 1;
d_T = dSS;
y_T = ySS;
x_T = xSS;
u_T = uSS;
ua_T = uaSS;
n_T = nSS;
theta_T = thetaSS;
lambda_T = lambdaSS; 
f_T = fSS; 
v_T = vSS; 
bg_T = bgSS; 
R_T = RSS;
Pi_T = PiSS; 
pStar_T = pStarSS;
h1P_T = 1/(1-xxiP*(muuSS/(RSS/PiSS)));
h2P_T = h1P_T; 
DeltaP_T = DeltaPSS; 
mc_T = mcSS; 
rk_T = rkSS;
q_T = 1;
w_T = wSS;
ttau_T = ttauSS; 
g_T = gSS;

obsgX_T = 0;
obsgC_T = 0;
obsgI_T = 0;
obsF_T = 0;
obsS_T = 0;
obsgWE_T = 0;
obsPi_T = 0;
obsFF_T = 0;
obsU_T = 0;


xHat_T = 0; 
cHat_T = 0;
invHat_T = 0;
nHat_T = 0;
PiHat_T = 0;


% Save endogenous variables back into ys
for ii = 1 : M_.orig_endo_nbr
  varname = deblank(M_.endo_names(ii,:));
  eval(['ys(' int2str(ii) ') = ' varname ';']); 
end

%fprintf('... Done!  Elapsed time: %2.2f seconds \n\n',toc(tStart))
