// Specifies HANK equations for counterFactuals.mod

//----------------------------------------------------------------
// Conditional expectation (#equations = nEpsilon * nAssets)
//----------------------------------------------------------------

@#for iEpsilon in 1 : nEpsilon

	@#for iAssets in 1 : nAssets
	
		// Compute conditional expectation
		# expectation_@{iEpsilon}_@{iAssets} = exp(0
		@#for iPower in 1 : nAssets
			+ expectationCoefficient_@{iEpsilon}_@{iPower} * expectationPoly_@{iAssets}_@{iPower}
		@#endfor
		);
		
		// Compute savings policy
		# assetsPrime_@{iEpsilon}_@{iAssets} = max((1 - ttau) * w * (bb * (1 - employmentGrid_@{iEpsilon}) + 
			employmentGrid_@{iEpsilon} * epsilonGrid_@{iEpsilon}) + (R(-1)/(Pi*muu)) * assetsGrid_@{iAssets} 
            - (expectation_@{iEpsilon}_@{iAssets} ^ (-1 / ssigma)),aaBar);
			
		// Compute next period's consumption
		@#for iEpsilonPrime in 1 : nEpsilon
		
			# expectationPrime_@{iEpsilonPrime}_@{iEpsilon}_@{iAssets} = exp(0
			@#for iPower in 1 : nAssets
				+ expectationCoefficient_@{iEpsilonPrime}_@{iPower}(1) * cos((@{iPower} - 1) * acos(min(max(
					2 * ((assetsPrime_@{iEpsilon}_@{iAssets} - assetsMin) / (assetsMax - assetsMin)) - 1,-1),1)))
			@#endfor
			); // cos and after - Chebyshev polynomials evaluated at assetsPrime
			
			# assetsPrime_@{iEpsilonPrime}_@{iEpsilon}_@{iAssets} = max((1 - ttau(1)) * w(1) * (bb * (1 - employmentGrid_@{iEpsilonPrime}) + 
				 employmentGrid_@{iEpsilonPrime} * epsilonGrid_@{iEpsilonPrime}) + (R/(Pi(1)*muu(1))) * assetsPrime_@{iEpsilon}_@{iAssets} - 
				(expectationPrime_@{iEpsilonPrime}_@{iEpsilon}_@{iAssets} ^ (-1 / ssigma)),aaBar); //assetsPrimePrime
			
			# consumptionPrime_@{iEpsilonPrime}_@{iEpsilon}_@{iAssets} = (1 - ttau(1)) * w(1) * (bb * (1 - employmentGrid_@{iEpsilonPrime}) + 
				 employmentGrid_@{iEpsilonPrime} * epsilonGrid_@{iEpsilonPrime}) + (R/(Pi(1)*muu(1))) * assetsPrime_@{iEpsilon}_@{iAssets} - 
				assetsPrime_@{iEpsilonPrime}_@{iEpsilon}_@{iAssets};
				
		@#endfor
				
		// Functional equation
		log(expectation_@{iEpsilon}_@{iAssets}) = log(bbetaL * (zeta(1)/zeta)* (R/(Pi(1)*muu(1)^ssigma)) * (0
		@#for iEpsilonPrime in 1 : nEpsilon
			+ epsilonTransition_@{iEpsilon}_@{iEpsilonPrime}(1) * (consumptionPrime_@{iEpsilonPrime}_@{iEpsilon}_@{iAssets} ^ 
				(-ssigma))
		@#endfor
		));
		
	@#endfor

@#endfor

//----------------------------------------------------------------
// Compute various objects over quadrature grid for integrating distribution
//----------------------------------------------------------------

@#for iEpsilon in 1 : nEpsilon

	@#for iAssets in 1 : nAssetsQuadrature
	
		// Compute conditional expectation
		# expectationQuadrature_@{iEpsilon}_@{iAssets} = exp(0
		@#for iPower in 1 : nAssets
			+ expectationCoefficient_@{iEpsilon}_@{iPower} * quadraturePoly_@{iAssets}_@{iPower}
		@#endfor
		);
		
		// Compute savings policy
		# assetsPrimeQuadrature_@{iEpsilon}_@{iAssets} = max((1 - ttau) * w * (bb * (1 - employmentGrid_@{iEpsilon}) + 
			employmentGrid_@{iEpsilon} * epsilonGrid_@{iEpsilon}) + (R(-1)/(Pi*muu)) * quadratureGrid_@{iAssets} - (expectationQuadrature_@{iEpsilon}_@{iAssets} ^ 
			(-1 / ssigma)),aaBar);
        
        // Compute consumption policy
        # consumptionQuadrature_@{iEpsilon}_@{iAssets} = (1 - ttau) * w * (bb * (1 - employmentGrid_@{iEpsilon}) + 
            employmentGrid_@{iEpsilon} * epsilonGrid_@{iEpsilon}) + (R(-1)/(Pi*muu)) * quadratureGrid_@{iAssets} - assetsPrimeQuadrature_@{iEpsilon}_@{iAssets};
			
		// Compute density away from borrowing constraint:
        // Distribution of households over quadratureGrid_@{iAssets}, 
        // where quadratureGrid_@{iAssets} is an asset position in period t (predetermined) 
		# measurePDF_@{iEpsilon}_@{iAssets} = exp(0 + measureCoefficient_@{iEpsilon}_1 * (quadratureGrid_@{iAssets} - 
			moment_@{iEpsilon}_1(-1))
			@#for iMoment in 2 : nMeasure
				+ measureCoefficient_@{iEpsilon}_@{iMoment} * ((quadratureGrid_@{iAssets} - moment_@{iEpsilon}_1(-1)) ^ @{iMoment} - 
				moment_@{iEpsilon}_@{iMoment}(-1))
			@#endfor
			);
			
	@#endfor
	
	// Total mass of distribution: 1/totalMass_@{iEpsilon} = g^0_{iEpsilon}
	# totalMass_@{iEpsilon} = 0
	@#for iAssets in 1 : nAssetsQuadrature
		+ quadratureWeights_@{iAssets} * measurePDF_@{iEpsilon}_@{iAssets}
	@#endfor
	;
	
@#endfor

//----------------------------------------------------------------
// Compute various objects at borrowing constraint for integrating distribution
//----------------------------------------------------------------

@#for iEpsilon in 1 : nEpsilon

	// Compute conditional expectation
	# expectationBC_@{iEpsilon} = exp(0
	@#for iPower in 1 : nAssets
		+ expectationCoefficient_@{iEpsilon}_@{iPower} * bcPoly_@{iPower}
	@#endfor
	);
	
	// Compute savings policy
	# assetsPrimeBC_@{iEpsilon} = max((1 - ttau) * w * (bb * (1 - employmentGrid_@{iEpsilon}) + 
		employmentGrid_@{iEpsilon} * epsilonGrid_@{iEpsilon}) + (R(-1)/(Pi*muu)) * aaBar - (expectationBC_@{iEpsilon} ^ 
		(-1 / ssigma)),aaBar);
		
@#endfor

//----------------------------------------------------------------
// Relationship between moments of distribution and parameters
// gives the value of measureCoefficient_@{iEpsilon}_@{iMoment} determined in period t-1 
// (#equations = nEpsilon * nMeasure)
//----------------------------------------------------------------

@#for iEpsilon in 1 : nEpsilon
	
	// First moments (uncentered)
	moment_@{iEpsilon}_1(-1) = (0
		@#for iAssets in 1 : nAssetsQuadrature
			+ quadratureWeights_@{iAssets} * quadratureGrid_@{iAssets} * measurePDF_@{iEpsilon}_@{iAssets}
		@#endfor
		) / totalMass_@{iEpsilon}; // 1/totalMass_@{iEpsilon} = g^0_{iEpsilon}
	
	// Higher order moments (centered)
	@#for iMoment in 2 : nMeasure
	moment_@{iEpsilon}_@{iMoment}(-1) = (0
		@#for iAssets in 1 : nAssetsQuadrature
			+ quadratureWeights_@{iAssets} * measurePDF_@{iEpsilon}_@{iAssets} * ((quadratureGrid_@{iAssets} - 
				moment_@{iEpsilon}_1(-1)) ^ @{iMoment})
		@#endfor
		) / totalMass_@{iEpsilon};
		
	@#endfor
		
@#endfor

//----------------------------------------------------------------
// Law of motion for density away from borrowing constraint 
// (#equations = nEpsilon * nMeasure)
//----------------------------------------------------------------

@#for iEpsilon in 1 : nEpsilon
	
	// First moment (uncentered)
	moment_@{iEpsilon}_1 = (0
	@#for iEpsilonTilde in 1 : nEpsilon
		+ ((1 - mHat_@{iEpsilonTilde}(-1)) * epsilonMass_@{iEpsilonTilde}(-1) * epsilonTransition_@{iEpsilonTilde}_@{iEpsilon} * (0
		@#for iAssets in 1 : nAssetsQuadrature
			+ quadratureWeights_@{iAssets} * measurePDF_@{iEpsilonTilde}_@{iAssets} *
				assetsPrimeQuadrature_@{iEpsilonTilde}_@{iAssets}
		@#endfor
		) / totalMass_@{iEpsilonTilde}) + mHat_@{iEpsilonTilde}(-1) * epsilonMass_@{iEpsilonTilde}(-1) * epsilonTransition_@{iEpsilonTilde}_@{iEpsilon} 
			* assetsPrimeBC_@{iEpsilonTilde}
	@#endfor
	) / epsilonMass_@{iEpsilon};
	
	// Higher order moments (uncentered)
	@#for iMoment in 2 : nMeasure
		moment_@{iEpsilon}_@{iMoment} = (0
		@#for iEpsilonTilde in 1 : nEpsilon
			+ ((1 - mHat_@{iEpsilonTilde}(-1)) * epsilonMass_@{iEpsilonTilde}(-1) * epsilonTransition_@{iEpsilonTilde}_@{iEpsilon} * (0
			@#for iAssets in 1 : nAssetsQuadrature
				+ quadratureWeights_@{iAssets} * measurePDF_@{iEpsilonTilde}_@{iAssets} * 
					(assetsPrimeQuadrature_@{iEpsilonTilde}_@{iAssets} - moment_@{iEpsilon}_1) ^ @{iMoment}
			@#endfor
			) / totalMass_@{iEpsilonTilde}) + mHat_@{iEpsilonTilde}(-1) * epsilonMass_@{iEpsilonTilde}(-1) * epsilonTransition_@{iEpsilonTilde}_@{iEpsilon}
				* (assetsPrimeBC_@{iEpsilonTilde} - moment_@{iEpsilon}_1) ^ @{iMoment}
		@#endfor
		) / epsilonMass_@{iEpsilon};
	@#endfor

@#endfor

//----------------------------------------------------------------
// Law of motion for mass at borrowing constraint
// (#equations = nEpsilon)
//----------------------------------------------------------------

@#for iEpsilon in 1 : nEpsilon

	mHat_@{iEpsilon} = (0
	@#for iEpsilonTilde in 1 : nEpsilon
		+ ((1 - mHat_@{iEpsilonTilde}(-1)) * epsilonMass_@{iEpsilonTilde}(-1) * epsilonTransition_@{iEpsilonTilde}_@{iEpsilon} * (0
		@#for iAssets in 1 : nAssetsQuadrature
			+ quadratureWeights_@{iAssets} * measurePDF_@{iEpsilonTilde}_@{iAssets} *
				(assetsPrimeQuadrature_@{iEpsilonTilde}_@{iAssets} <= aaBar + 1e-8)
		@#endfor
		) / totalMass_@{iEpsilonTilde}) + mHat_@{iEpsilonTilde}(-1) * epsilonMass_@{iEpsilonTilde}(-1) * epsilonTransition_@{iEpsilonTilde}_@{iEpsilon} 
			* (assetsPrimeBC_@{iEpsilonTilde} <= aaBar + 1e-8)
	@#endfor
	) / epsilonMass_@{iEpsilon};

@#endfor



//------------------------------------------------------------------------
// Time-varying transition probability (# equations = 4) 
//------------------------------------------------------------------------

epsilonTransition_1_1 = 1-f;
epsilonTransition_1_2 = f;
epsilonTransition_2_1 = rhox*(1-f);
epsilonTransition_2_2 = 1-rhox*(1-f);


//------------------------------------------------------------------------
// Expected impatient consumption  (# equations = nEpsilon + 1) 
//------------------------------------------------------------------------
@#for iEpsilon in 1 : nEpsilon
	
	momentC_@{iEpsilon}_1 = (0
		@#for iAssets in 1 : nAssetsQuadrature
			+ quadratureWeights_@{iAssets} * consumptionQuadrature_@{iEpsilon}_@{iAssets} * measurePDF_@{iEpsilon}_@{iAssets}
		@#endfor
		) / totalMass_@{iEpsilon}; // 1/totalMass_@{iEpsilon} = g^0_{iEpsilon}
				
@#endfor


cL = (0
    @#for iEpsilon in 1 : nEpsilon
        + epsilonMass_@{iEpsilon} * momentC_@{iEpsilon}_1
    @#endfor
	); 

//--------------------------------------------------------------------------
// Patient household optimization (# equations = 2) 
//--------------------------------------------------------------------------
  
1 = bbetaH * (R/(Pi(1)*muu(1)^ssigma)) * (zeta(1)/zeta) * (cH(1)/cH)^(-ssigma);

d = y - w * n  - kkappa * v - inv - (rrhouk*(1-ppsi)*(uk^(1/(1-ppsi))-1)) * (k(-1)/muu);  % dividends to households


//-----------------------------------------------------------------------------------------
// Labor market variables and the mass of idiosyncratic shocks (# equations = 7 + nEpsilon) 
//-----------------------------------------------------------------------------------------

n = (1-rhox) * n(-1) + MMBar * v * theta^(-ggamma); 

rhox = 1/(1+exp(rrhoxBar - rhoxtilde)); 

ua = u(-1) + rhox * n(-1);
 
u = (1-f) * (u(-1) + rhox * n(-1));

theta = v/ua;

u = 1 - n;
 
lambda = MMBar * theta^(-ggamma);

epsilonMass_1 = 1 - n;

epsilonMass_2 = n;


//------------------------------------------------------------------------
// Intermediate goods firm (# equations = 5)
//------------------------------------------------------------------------


kkappa/lambda =  (A^(1-aalpha) * (1-aalpha) * (uk*k(-1))^aalpha * n^(-aalpha) * mc - w)
                    + (muu(1)/(R/Pi(1))) * (1-rhox(1)) * kkappa/lambda(1);

q =  (1/(R/Pi(1))) * (rk(1)*uk(1) + q(1) * (1-ddelta) - (rrhouk*(1-ppsi)*(uk(1)^(1/(1-ppsi))-1)));

rk = rrhouk * uk^(ppsi/(1-ppsi));

1 = q * upsilon * (1 - (ss2prime/2) * ((inv*muu)/inv(-1) - muuSS)^2 - ss2prime * ((inv*muu)/inv(-1)-muuSS) * ((inv*muu)/inv(-1)))
    + ss2prime * (1/(R/Pi(1))) * q(1) * upsilon(1) * ((inv(1)*muu(1))/inv - muuSS) * ((inv(1)*muu(1))/inv)^2;

k = (1 - ddelta) * (k(-1)/muu) + upsilon * (1 - (ss2prime/2) * ((inv*muu)/inv(-1)-muuSS)^2) * inv;




//------------------------------------------------------------------------
// Wholesale firms optimization (# equations = 6)
//------------------------------------------------------------------------

DeltaP * y = (uk*k(-1))^aalpha * (A * n)^(1-aalpha) - FF; 

pStar = h1P/h2P; 
h1P = (1+etaP) * mc + xxiP * muu(1) * (1/(R/Pi(1))) * h1P(1);
h2P = 1 + xxiP * muu(1) * (1/(R/Pi(1))) * ((Pi^iiotaP * PiSS^(1-iiotaP))/Pi(1)) * h2P(1);
1 = (1-xxiP) * pStar^(-1/etaP) + xxiP * ((Pi(-1)^iiotaP * PiSS^(1-iiotaP))/Pi)^(-1/etaP);
DeltaP = (1-xxiP)*pStar^(-((1+etaP)/etaP)) + xxiP * ((Pi(-1)^iiotaP * PiSS^(1-iiotaP))/Pi)^(-((1+etaP)/etaP))*DeltaP(-1);




//----------------------------------------------------------------
// Prices (# equations = 5)
//----------------------------------------------------------------

rk = aalpha * (uk*k(-1))^(aalpha-1) * (A * n)^(1-aalpha) * mc;

# wwBar = wSS * (PiSS * muuSS)^(iiotaW/(1-iiotaW));
w = (w(-1)/(Pi*muu))^iiotaW * (wwBar * (n/nSS)^xxiW)^(1-iiotaW);

bg + ttau * (w * n + bb * w * u) = bb*w*u + g + (R(-1)/(Pi*muu)) * (bg(-1));

bg = bgSS;

TC = 0;

log(R/RSS) = rrhoR*log(R(-1)/RSS) + (1-rrhoR)*( pphiPi*log(Pi/PiStar) + pphiX*log(x/xSS) ) + ssigmaR*epsR;

//log(R/RSS) = rrhoR*log(R(-1)/RSS) + (1-rrhoR)*( pphiPi*((log(Pi/PiStar)+log(Pi(-1)/PiStar)+log(Pi(-2)/PiStar)+log(Pi(-3)/PiStar))/4) 
//+ pphiX*( (log((x*muu)/(x(-1)*muuSS))+log((x(-1)*muu(-1))/(x(-2)*muuSS))+log((x(-2)*muu(-2))/(x(-3)*muuSS)) + log((x(-3)*muu(-3))/(x(-4)*muuSS)))/4) )
//+ ssigmaR*epsR;

//--------------------------------------------------------------------------
// Definitions (# equations = 3) 
//-------------------------------------------------------------------------- 

c = Hshare * cH + (1 - Hshare) * cL;  

x = c + inv + g;

g = (1 - (1/gtilde)) * y;

//--------------------------------------------------------------------------
// Market clearing (# equations = 1) 
//-------------------------------------------------------------------------- 

y = c + inv + g + kkappa * v + (rrhouk*(1-ppsi)*(uk^(1/(1-ppsi))-1)) * (k(-1)/muu);

//----------------------------------------------------------------
// Law of motion for aggregate shocks (# equations = 8)
//----------------------------------------------------------------

log(1+etaP) = (1-rrhoetaP)*log(1+etaPSS) + rrhoetaP*log(1+etaP(-1)) + ssigmaetaP*epsetaP;
log(muu) = (1-rrhomuu)*log(muuSS) + rrhomuu*log(muu(-1)) + ssigmamuu*epsmuu;
log(upsilon) = rrhoupsilon*log(upsilon(-1)) + ssigmaupsilon*epsupsilon;
log(A) = rrhoTFP * log(A(-1)) + ssigmaTFP * epsTFP;
log(zeta) = rrhozeta * log(zeta(-1)) + ssigmazeta * epszeta;
log(PiStar) = (1-rrhopiStar)*log(PiSS) + rrhopiStar*log(PiStar(-1));
rhoxtilde = rrhorhox*rhoxtilde(-1) + ssigmarhox*epsrhox;
log(gtilde) = (1-rrhog)*log(gtildeSS) + rrhog*log(gtilde(-1)) + ssigmag*epsg;

//----------------------------------------------------------------
// Auxiliary variables of interest (# equations = 8)
//----------------------------------------------------------------

xHat = log(x/xSS); 
cHat = log(c/cSS);
cLHat = log(cL/cLSS);
cHHat = log(cH/cHSS);
invHat = log(inv/invSS);
nHat = n - nSS;
PiHat = log(Pi/PiSS);
vHat = log(v/vSS);


//----------------------------------------------------------------
// Observables in growth (# equations = 10)
//----------------------------------------------------------------
obsgX = log(x) - log(x(-1)) + log(muu/muuSS);
obsgC = log(c) - log(c(-1)) + log(muu/muuSS);
obsgI = log(inv) - log(inv(-1)) + log(muu/muuSS);
obsgG = log(g) - log(g(-1)) + log(muu/muuSS);
obsF = f - fSS;
obsS = rhox*(1-f)- rhoxSS*(1-fSS);
obsgWE = log(w) - log(w(-1)) + log(muu/muuSS) + log(Pi/PiSS) + ssigmaW*epsW;
obsPi = log(Pi/PiSS);
obsFF = log(R/RSS);
obsU = u - uSS;






