// Specifies equations for RANKDynamics.mod
//
// Daeha Cho, December, 2022


//--------------------------------------------------------------------------
// Patient household optimization (# equations = 2) 
//--------------------------------------------------------------------------
  
1 = bbetaH * (R/(Pi(1)*muu(1)^ssigma)) * (zeta(1)/zeta) * (cH(1)/cH)^(-ssigma);

d = y - w * n  - kkappa * v - inv - (rrhouk*(1-ppsi)*(uk^(1/(1-ppsi))-1)) * (k(-1)/muu);  % dividends to households


//-----------------------------------------------------------------------------------------
// Labor market variables and the mass of idiosyncratic shocks (# equations = 7) 
//-----------------------------------------------------------------------------------------

n = (1-rhox) * n(-1) + MMBar * v * theta^(-ggamma); 

rhox = 1/(1+exp(rrhoxBar - rhoxtilde)); 

ua = u(-1) + rhox * n(-1);
 
u = (1-f) * (u(-1) + rhox * n(-1));

theta = v/ua;

u = 1 - n;
 
lambda = MMBar * theta^(-ggamma);


//------------------------------------------------------------------------
// Intermediate goods firm (# equations = 5)
//------------------------------------------------------------------------


kkappa/lambda =  (A^(1-aalpha) * (1-aalpha) * (uk*k(-1))^aalpha * n^(-aalpha) * mc - w)
                    + (muu(1)/(R/Pi(1))) * (1-rhox(1)) * kkappa/lambda(1);

q =  (1/(R/Pi(1))) * (rk(1)*uk(1) + q(1) * (1-ddelta) - (rrhouk*(1-ppsi)*(uk(1)^(1/(1-ppsi))-1)));

rk = rrhouk * uk^(ppsi/(1-ppsi));

1 = q * upsilon * (1 - (ss2prime/2) * ((inv*muu)/inv(-1) - muuSS)^2 - ss2prime * ((inv*muu)/inv(-1)-muuSS) * ((inv*muu)/inv(-1)))
    + ss2prime * (1/(R/Pi(1))) * q(1) * upsilon(1) * ((inv(1)*muu(1))/inv - muuSS) * ((inv(1)*muu(1))/inv)^2;

k = (1 - ddelta) * (k(-1)/muu) + upsilon * (1 - (ss2prime/2) * ((inv*muu)/inv(-1)-muuSS)^2) * inv;




//------------------------------------------------------------------------
// Wholesale firms optimization (# equations = 6)
//------------------------------------------------------------------------

DeltaP * y = (uk*k(-1))^aalpha * (A * n)^(1-aalpha) - FF; 

pStar = h1P/h2P; 
h1P = (1+etaP) * mc + xxiP * muu(1) * (1/(R/Pi(1))) * h1P(1);
h2P = 1 + xxiP * muu(1) * (1/(R/Pi(1))) * ((Pi^iiotaP * PiSS^(1-iiotaP))/Pi(1)) * h2P(1);
1 = (1-xxiP) * pStar^(-1/etaP) + xxiP * ((Pi(-1)^iiotaP * PiSS^(1-iiotaP))/Pi)^(-1/etaP);
DeltaP = (1-xxiP)*pStar^(-((1+etaP)/etaP)) + xxiP * ((Pi(-1)^iiotaP * PiSS^(1-iiotaP))/Pi)^(-((1+etaP)/etaP))*DeltaP(-1);




//----------------------------------------------------------------
// Prices (# equations = 5)
//----------------------------------------------------------------

rk = aalpha * (uk*k(-1))^(aalpha-1) * (A * n)^(1-aalpha) * mc;

# wwBar = wSS * (PiSS * muuSS)^(iiotaW/(1-iiotaW));
w = (w(-1)/(Pi*muu))^iiotaW * (wwBar * (n/nSS)^xxiW)^(1-iiotaW);

bg + ttau * (w * n + bb * w * u) = bb*w*u + g + (R(-1)/(Pi*muu)) * (bg(-1));

bg = bgSS;

log(R/RSS) = rrhoR*log(R(-1)/RSS) + (1-rrhoR)*( pphiPi*log(Pi/PiStar) + pphiX*log(x/xSS) ) + ssigmaR*epsR;

//log(R/RSS) = rrhoR*log(R(-1)/RSS) + (1-rrhoR)*( pphiPi*((log(Pi/PiStar)+log(Pi(-1)/PiStar)+log(Pi(-2)/PiStar)+log(Pi(-3)/PiStar))/4) 
//+ pphiX*( (log((x*muu)/(x(-1)*muuSS))+log((x(-1)*muu(-1))/(x(-2)*muuSS))+log((x(-2)*muu(-2))/(x(-3)*muuSS)) + log((x(-3)*muu(-3))/(x(-4)*muuSS)))/4) )
//+ ssigmaR*epsR;

//--------------------------------------------------------------------------
// Definitions (# equations = 3) 
//-------------------------------------------------------------------------- 

c = cH;  

x = c + inv + g;

g = (1 - (1/gtilde)) * y;

//--------------------------------------------------------------------------
// Market clearing (# equations = 1) 
//-------------------------------------------------------------------------- 

y = c + inv + g + kkappa * v + (rrhouk*(1-ppsi)*(uk^(1/(1-ppsi))-1)) * (k(-1)/muu);

//----------------------------------------------------------------
// Law of motion for aggregate shocks (# equations = 9)
//----------------------------------------------------------------

log(1+etaP) = (1-rrhoetaP)*log(1+etaPSS) + rrhoetaP*log(1+etaP(-1)) + ssigmaetaP*epsetaP;
log(muu) = (1-rrhomuu)*log(muuSS) + rrhomuu*log(muu(-1)) + ssigmamuu*epsmuu;
log(upsilon) = rrhoupsilon*log(upsilon(-1)) + ssigmaupsilon*epsupsilon;
log(A) = rrhoTFP * log(A(-1)) + ssigmaTFP * epsTFP;
log(zeta) = rrhozeta * log(zeta(-1)) + ssigmazeta * epszeta;
log(PiStar) = (1-rrhopiStar)*log(PiSS) + rrhopiStar*log(PiStar(-1));
rhoxtilde = rrhorhox*rhoxtilde(-1) + ssigmarhox*epsrhox;
log(gtilde) = (1-rrhog)*log(gtildeSS) + rrhog*log(gtilde(-1)) + ssigmag*epsg;

//----------------------------------------------------------------
// Auxiliary variables of interest (# equations = 6)
//----------------------------------------------------------------

xHat = log(x/xSS); 
cHat = log(c/cSS);
invHat = log(inv/invSS);
nHat = n - nSS;
PiHat = log(Pi/PiSS);
fHat = f - fSS;


//----------------------------------------------------------------
// Observables in growth
//----------------------------------------------------------------
obsgX = log(x) - log(x(-1)) + log(muu/muuSS);
obsgC = log(c) - log(c(-1)) + log(muu/muuSS);
obsgI = log(inv) - log(inv(-1)) + log(muu/muuSS);
obsgG = log(g) - log(g(-1)) + log(muu/muuSS);
obsF = f - fSS;
obsS = rhox*(1-f)- rhoxSS*(1-fSS);
obsgWE = log(w) - log(w(-1)) + log(muu/muuSS) + log(Pi/PiSS) + ssigmaW*epsW;
obsPi = log(Pi/PiSS);
obsFF = log(R/RSS);
obsU = u - uSS;


