% Computes the steady state of HANK under alternative calibrations
%
% Daeha Cho, December, 2022


%----------------------------------------------------------------
% Set parameters
%----------------------------------------------------------------
load('HANKparam')
setParameters;

if calibrationType == 1 
    cUOvercEMean = 1-0.3/(1-Hshare);
elseif calibrationType == 2  
    cUOvercEMean = 1-0.1/(1-Hshare);
    assetsMax = 0.3;
elseif calibrationType == 3 
    if MPCType == 1
    [];
    elseif MPCType == 2
    targetMPC = 0.1;
    assetsMax = 6;
    end
elseif calibrationType == 4 
    bgSS = xSS * 4* 0.35;          % liquid assets  
    ttauSS = (bInit * wSS * uSS  + gSS + (RSS/(PiSS*muuSS)-1) * bgSS)/(wSS*nSS + bInit*wSS*uSS);
elseif calibrationType == 5
    if nMeasureType == 2
    nMeasure = 2;
    elseif nMeasureType == 3
    nMeasure = 3;
    elseif nMeasureType == 4
    nMeasure = 4;
    end
end
    


%----------------------------------------------------------------
% Compute Steady State
%----------------------------------------------------------------

% Solve for steady state capital stock, distribution, and decision rules
coreSteadyState;

% % Compute decision rules along fine grid for analysis
rootParameters = [bbetaL bb]';
[~,~,~,mConsumption] = computeMCResidualHistogram(rootParameters);

% Load the histogram-based distribution and consumption decision rule 
 load steadyState_Decision_Distribution mHistogram_hist mConsumption_hist

% Compute density along fine grid
mDistributionFine = zeros(nEpsilon,nAssetsFine);
for iEpsilon = 1 : nEpsilon

	% First moment (uncentered)
	mGridMoments = zeros(nAssetsFine,nMeasure);
	mGridMoments(:,1) = (vAssetsGridFine - mMoments(iEpsilon,1));
	
	% Higher order moments (centered)
	for iMoment = 2 : nMeasure
		mGridMoments(:,iMoment) = (vAssetsGridFine - mMoments(iEpsilon,1)) .^ iMoment - ...
			mMoments(iEpsilon,iMoment);
	end
	
	% Compute density away from borrowing constraint
	mDistributionFine(iEpsilon,:) = mParameters(iEpsilon,1) * exp(mGridMoments * ...
		mParameters(iEpsilon,2:nMeasure+1)');
		
	% Mass at borrowing constraint
	%mDistributionFine(iEpsilon,1) = mHat(iEpsilon,1); % Commented out for now; need fine quadrature grid to capture correctly
		
end





%----------------------------------------------------------------
% Save parameters in .mat files to import into Dynare 
%----------------------------------------------------------------

load('steadyState_Decision_Distribution');

    
% Economic parameters
save economicParameters.mat bbetaL ssigma aaBar bbetaH Hshare Hshare_T aalpha ddelta ss2prime rrhouk ppsi ...
     xxiP iiotaP ggamma rrhoxBar kkappa MMBar bb iiotaW xxiW FF rrhoR pphiPi pphiX vEpsilonGrid ...
     rrhoetaP rrhomuu rrhoupsilon rrhoTFP rrhozeta rrhorhox rrhopiStar rrhog... 
     ssigmaetaP ssigmamuu ssigmaupsilon ssigmaTFP ssigmaR ssigmazeta ssigmarhox...
     ssigmapiStar ssigmag ssigmaW;
    
% Steady state aggregate variables
save steadyStateAggregates.mat RSS PiSS muuSS nSS fSS rhoxSS wSS etaPSS DeltaPSS pStarSS h1PSS h2PSS...
     mcSS rkSS kSS lambdaSS uSS uaSS thetaSS vSS xSS bgSS ttauSS dSS ySS invSS gSS gtildeSS cSS cHSS cLSS cH_TSS cL_TSS aHSS...
     mEpsilonTransitionSS vEpsilonInvariant

% Approximation parameters
save approximationParameters.mat nEpsilon nAssets nState assetsMin assetsMax nAssetsFine nStateFine nAssetsQuadrature nStateQuadrature ...
	 nMeasure nMeasureCoefficients maxIterations tolerance dampening

% Grids
save grids.mat vAssetsGridZeros vAssetsGrid vEmploymentGrid vEpsilonGrid mEmploymentGrid mAssetsGrid mEmploymentPrimeGrid vAssetsGridFine ...
	 vAssetsGridFineZeros mEmploymentGridFine mAssetsGridFine mEmploymentPrimeGridFine vQuadratureWeights ...
	 vAssetsGridQuadratureZeros vAssetsGridQuadrature mEmploymentGridQuadrature mAssetsGridQuadrature
		
% Polynomials
save polynomials.mat vAssetsPoly vAssetsPolySquared vAssetsPolyFine vAssetsPolyQuadrature vAssetsPolyBC


