// Declare variables for counterFactuals.mod
//
// Daeha Cho, December, 2022

//----------------------------------------------------------------
// Conditional expectation coefficients
//----------------------------------------------------------------

@#for iEpsilon in 1 : nEpsilon
    @#for iPower in 1 : nAssets
        var expectationCoefficient_@{iEpsilon}_@{iPower};
    @#endfor
@#endfor

//----------------------------------------------------------------
// Density of households away from borrowing constraint
//----------------------------------------------------------------

// Moments of the distribution
@#for iEpsilon in 1 : nEpsilon
    @#for iMoment in 1 : nMeasure
        var moment_@{iEpsilon}_@{iMoment};
    @#endfor
@#endfor

// Parameters of the distribution
@#for iEpsilon in 1 : nEpsilon
    @#for iParameter in 1 : nMeasure
        var measureCoefficient_@{iEpsilon}_@{iParameter};  
    @#endfor
@#endfor

// Consumption moments
@#for iEpsilon in 1 : nEpsilon
    var momentC_@{iEpsilon}_1;
@#endfor

//----------------------------------------------------------------
// Mass at borrowing constraint
//----------------------------------------------------------------

@#for iEpsilon in 1 : nEpsilon
    var mHat_@{iEpsilon};
@#endfor

//----------------------------------------------------------------
// Epsilon transition matrix
//----------------------------------------------------------------

@#for iEpsilon in 1 : nEpsilon
	@#for iEpsilonPrime in 1 : nEpsilon
		var epsilonTransition_@{iEpsilon}_@{iEpsilonPrime};
	@#endfor
@#endfor

//----------------------------------------------------------------
// Mass of idiosyncratic shocks
//----------------------------------------------------------------

@#for iEpsilon in 1 : nEpsilon
    var epsilonMass_@{iEpsilon};
@#endfor

//----------------------------------------------------------------
// Quantities
//----------------------------------------------------------------

var c cL cH inv k uk d y x u ua n theta lambda f v rhox bg g TC; 
  
 
//----------------------------------------------------------------
// Prices
//----------------------------------------------------------------

var R Pi pStar h1P h2P DeltaP mc rk q w ttau; 

//----------------------------------------------------------------
// Aggregate shocks
//----------------------------------------------------------------

var etaP muu upsilon A zeta PiStar rhoxtilde gtilde;

//----------------------------------------------------------------
// Auxiliary variables we're interested in
//----------------------------------------------------------------

var xHat cHat cLHat cHHat invHat nHat PiHat vHat;

//----------------------------------------------------------------
// Observables
//----------------------------------------------------------------

var obsgX obsgC obsgI obsgG obsF obsS obsgWE obsPi obsFF obsU;

//----------------------------------------------------------------
// RANK variables
//----------------------------------------------------------------

var c_R cH_R inv_R k_R uk_R d_R y_R x_R u_R ua_R n_R theta_R lambda_R f_R v_R bg_R g_R; 
  
var R_R Pi_R pStar_R h1P_R h2P_R DeltaP_R mc_R rk_R q_R w_R ttau_R; 

var obsgX_R obsgC_R obsgI_R obsF_R obsS_R obsgWE_R obsPi_R obsFF_R obsU_R;

var xHat_R cHat_R invHat_R nHat_R PiHat_R;

//----------------------------------------------------------------
// TANK variables
//----------------------------------------------------------------

var c_T cH_T cL_T inv_T k_T uk_T d_T y_T x_T u_T ua_T n_T theta_T lambda_T f_T v_T bg_T g_T; 
  
var R_T Pi_T pStar_T h1P_T h2P_T DeltaP_T mc_T rk_T q_T w_T ttau_T; 

var obsgX_T obsgC_T obsgI_T obsF_T obsS_T obsgWE_T obsPi_T obsFF_T obsU_T;

var xHat_T cHat_T invHat_T nHat_T PiHat_T;

//----------------------------------------------------------------
// Shocks
//----------------------------------------------------------------

varexo epsetaP epsmuu epsupsilon epsTFP epsR epszeta epsrhox  epsg epsW;
